@@ -65,6 +65,7 @@
 #include "control.h"
 #include "manager.h"
 #include "sdpd.h"
+#include "telephony.h"
 
 typedef enum {
 	HEADSET	= 1 << 0,
@@ -463,13 +464,11 @@ static void auth_cb(DBusError *derr, void *user_data)
 	} else {
 		char hs_address[18];
 
-		headset_set_authorized(device);
-
 		ba2str(&device->dst, hs_address);
-
 		debug("Accepted headset connection from %s for %s",
 						hs_address, device->path);
-		headset_set_state(device, HEADSET_STATE_CONNECTED);
+
+		headset_set_authorized(device);
 	}
 }
 
@@ -510,16 +509,16 @@ static void ag_io_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 		goto drop;
 	}
 
+	headset_set_state(device, HEADSET_STATE_CONNECT_IN_PROGRESS);
+
 	err = btd_request_authorization(&device->src, &device->dst, uuid,
 				auth_cb, device);
 	if (err < 0) {
 		debug("Authorization denied: %s", strerror(-err));
-		headset_close_rfcomm(device);
+		headset_set_state(device, HEADSET_STATE_DISCONNECTED);
 		return;
 	}
 
-	headset_set_state(device, HEADSET_STATE_CONNECT_IN_PROGRESS);
-
 	return;
 
 drop:
@@ -672,7 +671,7 @@ static int gateway_server_init(struct audio_adapter *adapter)
 
 static int audio_probe(struct btd_device *device, GSList *records)
 {
-	struct adapter *adapter = device_get_adapter(device);
+	struct btd_adapter *adapter = device_get_adapter(device);
 	const gchar *path = device_get_path(device);
 	const char *source, *destination;
 	bdaddr_t src, dst;
@@ -725,7 +724,7 @@ static struct audio_adapter *create_audio_adapter(const char *path, bdaddr_t *sr
 	return adp;
 }
 
-static struct audio_adapter *get_audio_adapter(struct adapter *adapter)
+static struct audio_adapter *get_audio_adapter(struct btd_adapter *adapter)
 {
 	struct audio_adapter *adp;
 	const gchar *path = adapter_get_path(adapter);
@@ -746,7 +745,7 @@ static struct audio_adapter *get_audio_adapter(struct adapter *adapter)
 	return adp;
 }
 
-static int headset_server_probe(struct adapter *adapter)
+static int headset_server_probe(struct btd_adapter *adapter)
 {
 	struct audio_adapter *adp;
 	const gchar *path = adapter_get_path(adapter);
@@ -760,7 +759,7 @@ static int headset_server_probe(struct adapter *adapter)
 	return headset_server_init(adp);
 }
 
-static void headset_server_remove(struct adapter *adapter)
+static void headset_server_remove(struct btd_adapter *adapter)
 {
 	struct audio_adapter *adp;
 	const gchar *path = adapter_get_path(adapter);
@@ -792,7 +791,7 @@ static void headset_server_remove(struct adapter *adapter)
 	}
 }
 
-static int gateway_server_probe(struct adapter *adapter)
+static int gateway_server_probe(struct btd_adapter *adapter)
 {
 	struct audio_adapter *adp;
 	const gchar *path = adapter_get_path(adapter);
@@ -806,7 +805,7 @@ static int gateway_server_probe(struct adapter *adapter)
 	return gateway_server_init(adp);
 }
 
-static void gateway_server_remove(struct adapter *adapter)
+static void gateway_server_remove(struct btd_adapter *adapter)
 {
 	struct audio_adapter *adp;
 	const gchar *path = adapter_get_path(adapter);
@@ -828,7 +827,7 @@ static void gateway_server_remove(struct adapter *adapter)
 	}
 }
 
-static int a2dp_server_probe(struct adapter *adapter)
+static int a2dp_server_probe(struct btd_adapter *adapter)
 {
 	struct audio_adapter *adp;
 	const gchar *path = adapter_get_path(adapter);
@@ -842,7 +841,7 @@ static int a2dp_server_probe(struct adapter *adapter)
 	return a2dp_init(connection, config);
 }
 
-static int avrcp_server_probe(struct adapter *adapter)
+static int avrcp_server_probe(struct btd_adapter *adapter)
 {
 	struct audio_adapter *adp;
 	const gchar *path = adapter_get_path(adapter);
@@ -932,8 +931,10 @@ int audio_manager_init(DBusConnection *conn, GKeyFile *conf)
 	g_strfreev(list);
 
 proceed:
-	if (enabled.headset)
+	if (enabled.headset) {
+		telephony_init();
 		btd_register_adapter_driver(&headset_server_driver);
+	}
 
 	if (enabled.gateway)
 		btd_register_adapter_driver(&gateway_server_driver);
@@ -956,8 +957,10 @@ void audio_manager_exit(void)
 	if (config)
 		g_key_file_free(config);
 
-	if (enabled.headset)
+	if (enabled.headset) {
 		btd_unregister_adapter_driver(&headset_server_driver);
+		telephony_exit();
+	}
 
 	if (enabled.gateway)
 		btd_unregister_adapter_driver(&gateway_server_driver);
