@@ -41,11 +41,10 @@
 #include <dbus/dbus.h>
 #include <gdbus.h>
 
-#include "hcid.h"
-
 #include "logging.h"
 #include "textfile.h"
 
+#include "hcid.h"
 #include "adapter.h"
 #include "device.h"
 #include "dbus-common.h"
@@ -54,6 +53,7 @@
 #include "glib-helper.h"
 #include "agent.h"
 #include "sdp-xml.h"
+#include "storage.h"
 
 #define DEFAULT_XML_BUF_SIZE	1024
 #define DISCONNECT_TIMER	2
@@ -66,7 +66,7 @@ struct btd_driver_data {
 struct btd_device {
 	gchar		*address;
 	gchar		*path;
-	struct adapter	*adapter;
+	struct btd_adapter	*adapter;
 	GSList		*uuids;
 	GSList		*drivers;		/* List of driver_data */
 	gboolean	temporary;
@@ -87,6 +87,7 @@ struct browse_req {
 	struct btd_device *device;
 	GSList *uuids_added;
 	GSList *uuids_removed;
+	sdp_list_t *records;
 	int search_uuid;
 	gboolean browse;
 };
@@ -122,7 +123,7 @@ static void device_free(gpointer user_data)
 
 static gboolean device_is_paired(struct btd_device *device)
 {
-	struct adapter *adapter = device->adapter;
+	struct btd_adapter *adapter = device->adapter;
 	char filename[PATH_MAX + 1], *str;
 	gboolean ret;
 	const gchar *source = adapter_get_address(adapter);
@@ -136,35 +137,23 @@ static gboolean device_is_paired(struct btd_device *device)
 	return ret;
 }
 
-static char *device_get_name(struct btd_device *device)
-{
-	struct adapter *adapter = device->adapter;
-	char filename[PATH_MAX + 1];
-	const gchar *source = adapter_get_address(adapter);
-
-	create_name(filename, PATH_MAX, STORAGEDIR, source, "names");
-	return textfile_caseget(filename, device->address);
-}
-
 static DBusMessage *get_properties(DBusConnection *conn,
 				DBusMessage *msg, void *user_data)
 {
 	struct btd_device *device = user_data;
-	struct adapter *adapter = device->adapter;
+	struct btd_adapter *adapter = device->adapter;
 	DBusMessage *reply;
 	DBusMessageIter iter;
 	DBusMessageIter dict;
 	bdaddr_t src, dst;
-	char path[MAX_PATH_LENGTH];
-	char buf[64];
-	const char *ptr;
-	char *name, *ppath, **uuids;
+	char name[248];
+	char **uuids;
+	const char *ptr, *source;
 	dbus_bool_t boolean;
 	uint32_t class;
 	int i;
 	GSList *l;
-	uint16_t dev_id = adapter_get_dev_id(adapter);
-	const gchar *source = adapter_get_address(adapter);
+	struct active_conn_info *dev;
 
 	reply = dbus_message_new_method_return(msg);
 	if (!reply)
@@ -182,12 +171,24 @@ static DBusMessage *get_properties(DBusConnection *conn,
 			&device->address);
 
 	/* Name */
-	name = device_get_name(device);
-	if (name) {
+	ptr = NULL;
+	memset(name, 0, sizeof(name));
+	source = adapter_get_address(adapter);
+
+	if (read_device_name(source, device->address, name) == 0) {
+		ptr = name;
 		dbus_message_iter_append_dict_entry(&dict, "Name",
-				DBUS_TYPE_STRING, &name);
+				DBUS_TYPE_STRING, &ptr);
 	}
 
+	if (read_device_alias(source, device->address, name, sizeof(name)) > 0)
+		ptr = name;
+
+	/* Alias: use Name if Alias doesn't exist */
+	if (ptr)
+		dbus_message_iter_append_dict_entry(&dict, "Alias",
+				DBUS_TYPE_STRING, &ptr);
+
 	str2ba(source, &src);
 	str2ba(device->address, &dst);
 
@@ -197,17 +198,6 @@ static DBusMessage *get_properties(DBusConnection *conn,
 				DBUS_TYPE_UINT32, &class);
 	}
 
-	/* Alias */
-	if (get_device_alias(dev_id, &dst, buf, sizeof(buf)) > 0) {
-		ptr = buf;
-		dbus_message_iter_append_dict_entry(&dict, "Alias",
-				DBUS_TYPE_STRING, &ptr);
-	} else if (name) {
-		dbus_message_iter_append_dict_entry(&dict, "Alias",
-				DBUS_TYPE_STRING, &name);
-		free(name);
-	}
-
 	/* Paired */
 	boolean = device_is_paired(device);
 	dbus_message_iter_append_dict_entry(&dict, "Paired",
@@ -219,8 +209,8 @@ static DBusMessage *get_properties(DBusConnection *conn,
 			DBUS_TYPE_BOOLEAN, &boolean);
 
 	/* Connected */
-	if (g_slist_find_custom(adapter->active_conn, &dst,
-				active_conn_find_by_bdaddr))
+	dev = adapter_search_active_conn_by_bdaddr(adapter, &dst);
+	if (dev)
 		boolean = TRUE;
 	else
 		boolean = FALSE;
@@ -237,60 +227,34 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	g_free(uuids);
 
 	/* Adapter */
-	snprintf(path, sizeof(path), "%s/hci%d", "/org/bluez", dev_id);
-	ppath = path;
+	ptr = adapter_get_path(adapter);
 	dbus_message_iter_append_dict_entry(&dict, "Adapter",
-			DBUS_TYPE_OBJECT_PATH, &ppath);
+			DBUS_TYPE_OBJECT_PATH, &ptr);
 
 	dbus_message_iter_close_container(&iter, &dict);
 
 	return reply;
 }
 
-static int remove_device_alias(const char *source, const char *destination)
-{
-	char filename[PATH_MAX + 1];
-
-	create_name(filename, PATH_MAX, STORAGEDIR, source, "aliases");
-	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-
-	return textfile_del(filename, destination);
-}
-
 static DBusMessage *set_alias(DBusConnection *conn, DBusMessage *msg,
 					const char *alias, void *data)
 {
 	struct btd_device *device = data;
-	struct adapter *adapter = device->adapter;
-	bdaddr_t bdaddr;
-	int ecode;
-	char *str, filename[PATH_MAX + 1];
-	uint16_t dev_id = adapter_get_dev_id(adapter);
+	struct btd_adapter *adapter = device->adapter;
 	const gchar *source = adapter_get_address(adapter);
-
-	str2ba(device->address, &bdaddr);
+	int err;
 
 	/* Remove alias if empty string */
-	if (g_str_equal(alias, "")) {
-		create_name(filename, PATH_MAX, STORAGEDIR, source,
-				"names");
-		str = textfile_caseget(filename, device->address);
-		ecode = remove_device_alias(source, device->address);
-	} else {
-		str = g_strdup(alias);
-		ecode = set_device_alias(dev_id, &bdaddr, alias);
-	}
-
-	if (ecode < 0)
+	err = write_device_alias(source, device->address,
+			g_str_equal(alias, "") ? NULL : alias);
+	if (err < 0)
 		return g_dbus_create_error(msg,
 				ERROR_INTERFACE ".Failed",
-				strerror(-ecode));
+				strerror(-err));
 
 	dbus_connection_emit_property_changed(conn, dbus_message_get_path(msg),
 					DEVICE_INTERFACE, "Alias",
-					DBUS_TYPE_STRING, &str);
-
-	g_free(str);
+					DBUS_TYPE_STRING, &alias);
 
 	return dbus_message_new_method_return(msg);
 }
@@ -299,13 +263,10 @@ static DBusMessage *set_trust(DBusConnection *conn, DBusMessage *msg,
 					dbus_bool_t value, void *data)
 {
 	struct btd_device *device = data;
-	struct adapter *adapter = device->adapter;
-	bdaddr_t local;
+	struct btd_adapter *adapter = device->adapter;
 	const gchar *source = adapter_get_address(adapter);
 
-	str2ba(source, &local);
-
-	write_trust(&local, device->address, GLOBAL_TRUST, value);
+	write_trust(source, device->address, GLOBAL_TRUST, value);
 
 	dbus_connection_emit_property_changed(conn, dbus_message_get_path(msg),
 					DEVICE_INTERFACE, "Trusted",
@@ -350,7 +311,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		return set_trust(conn, msg, value, data);
 	} else if (g_str_equal("Alias", property)) {
-		char *alias;
+		const char *alias;
 
 		if (dbus_message_iter_get_arg_type(&sub) != DBUS_TYPE_STRING)
 			return invalid_args(msg);
@@ -365,7 +326,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 static void discover_services_req_exit(void *user_data)
 {
 	struct btd_device *device = user_data;
-	struct adapter *adapter = device->adapter;
+	struct btd_adapter *adapter = device->adapter;
 	bdaddr_t src, dst;
 	const gchar *source = adapter_get_address(adapter);
 
@@ -418,7 +379,7 @@ static DBusMessage *cancel_discover(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct btd_device *device = user_data;
-	struct adapter *adapter = device->adapter;
+	struct btd_adapter *adapter = device->adapter;
 	bdaddr_t src, dst;
 	const gchar *source = adapter_get_address(adapter);
 
@@ -449,7 +410,6 @@ static gboolean disconnect_timeout(gpointer user_data)
 {
 	struct btd_device *device = user_data;
 	struct active_conn_info *ci;
-	GSList *l;
 	disconnect_cp cp;
 	bdaddr_t bda;
 	int dd;
@@ -458,12 +418,12 @@ static gboolean disconnect_timeout(gpointer user_data)
 	device->disconn_timer = 0;
 
 	str2ba(device->address, &bda);
-	l = g_slist_find_custom(device->adapter->active_conn,
-				&bda, active_conn_find_by_bdaddr);
-	if (!l)
+
+	ci = adapter_search_active_conn_by_bdaddr(device->adapter, &bda);
+
+	if (!ci)
 		return FALSE;
 
-	ci = l->data;
 	dd = hci_open_dev(dev_id);
 	if (dd < 0)
 		goto fail;
@@ -485,13 +445,14 @@ static DBusMessage *disconnect(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct btd_device *device = user_data;
-	GSList *l;
 	bdaddr_t bda;
+	struct active_conn_info *dev;
 
 	str2ba(device->address, &bda);
-	l = g_slist_find_custom(device->adapter->active_conn,
-				&bda, active_conn_find_by_bdaddr);
-	if (!l)
+
+	dev = adapter_search_active_conn_by_bdaddr(device->adapter, &bda);
+
+	if (!dev)
 		return g_dbus_create_error(msg,
 				ERROR_INTERFACE ".NotConnected",
 				"Device is not connected");
@@ -522,7 +483,7 @@ static GDBusSignalTable device_signals[] = {
 	{ }
 };
 
-struct btd_device *device_create(DBusConnection *conn, struct adapter *adapter,
+struct btd_device *device_create(DBusConnection *conn, struct btd_adapter *adapter,
 					const gchar *address)
 {
 	gchar *address_up;
@@ -659,7 +620,7 @@ void device_probe_drivers(struct btd_device *device, GSList *uuids, sdp_list_t *
 
 void device_remove_drivers(struct btd_device *device, GSList *uuids, sdp_list_t *recs)
 {
-	struct adapter *adapter = device_get_adapter(device);
+	struct btd_adapter *adapter = device_get_adapter(device);
 	const gchar *src = adapter_get_address(adapter);
 	const gchar *dst = device_get_address(device);
 	GSList *list;
@@ -806,16 +767,25 @@ static void services_changed(struct browse_req *req)
 	g_free(uuids);
 }
 
+static int rec_cmp(const void *a, const void *b)
+{
+	const sdp_record_t *r1 = a;
+	const sdp_record_t *r2 = b;
+
+	return r1->handle - r2->handle;
+}
+
 static void update_services(struct browse_req *req, sdp_list_t *recs)
 {
 	struct btd_device *device = req->device;
-	struct adapter *adapter = device_get_adapter(device);
+	struct btd_adapter *adapter = device_get_adapter(device);
 	const gchar *src = adapter_get_address(adapter);
 	const gchar *dst = device_get_address(device);
 	sdp_list_t *seq;
 
 	for (seq = recs; seq; seq = seq->next) {
 		sdp_record_t *rec = (sdp_record_t *) seq->data;
+		sdp_buf_t pdu;
 		sdp_list_t *svcclass = NULL;
 		gchar *uuid_str;
 		GSList *l;
@@ -825,13 +795,28 @@ static void update_services(struct browse_req *req, sdp_list_t *recs)
 
 		if (sdp_get_service_classes(rec, &svcclass) < 0)
 			continue;
-		store_record(src, dst, rec);
 
 		/* Extract the first element and skip the remainning */
 		uuid_str = bt_uuid2string(svcclass->data);
 		if (!uuid_str)
 			continue;
 
+		/* Check for duplicates */
+		if (sdp_list_find(req->records, rec, rec_cmp))
+			continue;
+
+		store_record(src, dst, rec);
+
+		/* Copy record */
+		if (sdp_gen_record_pdu(rec, &pdu) == 0) {
+			sdp_record_t *record;
+			int scanned;
+
+			record = sdp_extract_pdu(pdu.data, pdu.data_size,
+						&scanned);
+			req->records = sdp_list_append(req->records, record);
+		}
+
 		l = g_slist_find_custom(device->uuids, uuid_str,
 				(GCompareFunc) strcmp);
 		if (!l)
@@ -849,7 +834,7 @@ static void update_services(struct browse_req *req, sdp_list_t *recs)
 
 static void store(struct btd_device *device)
 {
-	struct adapter *adapter = device->adapter;
+	struct btd_adapter *adapter = device->adapter;
 	bdaddr_t src, dst;
 	char *str;
 	const gchar *source = adapter_get_address(adapter);
@@ -871,14 +856,17 @@ static void browse_cb(sdp_list_t *recs, int err, gpointer user_data)
 {
 	struct browse_req *req = user_data;
 	struct btd_device *device = req->device;
-	struct adapter *adapter = device->adapter;
+	struct btd_adapter *adapter = device->adapter;
 	bdaddr_t src, dst;
 	uuid_t uuid;
 	DBusMessage *reply;
 	const gchar *source = adapter_get_address(adapter);
 
-	if (err < 0)
+	if (err < 0) {
+		error("%s: error updating services: %s (%d)",
+				device->path, strerror(-err), -err);
 		goto proceed;
+	}
 
 	update_services(req, recs);
 
@@ -896,27 +884,30 @@ static void browse_cb(sdp_list_t *recs, int err, gpointer user_data)
 
 probe:
 
-	if (!req->uuids_added && !req->uuids_removed)
+	if (!req->uuids_added && !req->uuids_removed) {
+		debug("%s: No service found", device->path);
 		goto proceed;
+	}
 
 	/* Probe matching drivers for services added */
 	if (req->uuids_added)
-		device_probe_drivers(device, req->uuids_added, recs);
+		device_probe_drivers(device, req->uuids_added, req->records);
 
 	/* Remove drivers for services removed */
 	if (req->uuids_removed)
-		device_remove_drivers(device, req->uuids_removed, recs);
-
-	/* Store the device's profiles in the filesystem */
-	store(device);
+		device_remove_drivers(device, req->uuids_removed, req->records);
 
 	/* Propagate services changes */
 	services_changed(req);
 
 proceed:
+
+	/* Store the device's profiles in the filesystem */
+	store(device);
+
 	if (dbus_message_is_method_call(req->msg, DEVICE_INTERFACE,
 			"DiscoverServices")) {
-		discover_device_reply(req, recs);
+		discover_device_reply(req, req->records);
 		goto cleanup;
 	}
 
@@ -946,20 +937,19 @@ cleanup:
 		device->discov_requestor = NULL;
 	}
 
-	if (recs != NULL)
-		sdp_list_free(recs, (sdp_free_func_t) sdp_record_free);
-
 	dbus_message_unref(req->msg);
 	dbus_connection_unref(req->conn);
 	g_slist_free(req->uuids_added);
 	g_slist_free(req->uuids_removed);
+	if (req->records)
+		sdp_list_free(req->records, (sdp_free_func_t) sdp_record_free);
 	g_free(req);
 }
 
 int device_browse(struct btd_device *device, DBusConnection *conn,
 			DBusMessage *msg, uuid_t *search)
 {
-	struct adapter *adapter = device->adapter;
+	struct btd_adapter *adapter = device->adapter;
 	struct browse_req *req;
 	bdaddr_t src, dst;
 	uuid_t uuid;
@@ -997,7 +987,7 @@ int device_browse(struct btd_device *device, DBusConnection *conn,
 	return bt_search_service(&src, &dst, &uuid, browse_cb, req, NULL);
 }
 
-struct adapter *device_get_adapter(struct btd_device *device)
+struct btd_adapter *device_get_adapter(struct btd_device *device)
 {
 	if (!device)
 		return NULL;
